@@ -12,7 +12,7 @@ use base qw(Net::DNS::Resolver::Base DynaLoader), OS_CONF;
 our $VERSION;
 
 BEGIN {
-	$VERSION = '1.25_02';
+	$VERSION = '1.25_03';
 	eval { __PACKAGE__->bootstrap($VERSION) };
 }
 
@@ -101,13 +101,12 @@ sub new {
 	my ( $class, @args ) = @_;
 	my $self = $class->SUPER::new();
 	$self->nameservers( $self->SUPER::nameservers );
-	$self->_finalise_config;				# base configuration
+	$self->_finalise_config;				# default configuration
 	$self->{update} = {} if @args;				# force context rebuild
 	while ( my $attr = shift @args ) {
 		my $value = shift @args;
 		$self->$attr( ref($value) ? @$value : $value );
 	}
-	$self->_finalise_config;
 	return $self;
 }
 
@@ -137,10 +136,11 @@ By default, DNS queries are sent to the IP addresses listed in
 sub nameservers {
 	my ( $self, @nameservers ) = @_;
 	if ( defined wantarray ) {
-		my $config   = $self->{config};			# merge config updates
+		my $config   = $self->{config};
 		my $update   = $self->{update};
-		my ($setfwd) = grep {$_} $update->{set_fwd}, $config->{set_fwd};
-		my @value    = map  { ref($_) ? @$_ : $_ } $setfwd;
+		my @setfwd   = ( $update->{set_fwd}, $config->{set_fwd}, [] );
+		my ($setfwd) = grep { defined $_ } @setfwd;
+		my @value    = map { ref($_) ? @$_ : $_ } $setfwd;
 		return @value;
 	}
 	$self->set_fwd() unless @nameservers;
@@ -456,19 +456,19 @@ END
 	$self->{update} ||= {};					# force config rebuild
 	$self->_finalise_config;
 	my %config = %{$self->{config}};
-	my $optref = $config{set_option} || [];			# extract option hash
-	my %option = map {@$_} @$optref;
-	my @option;
+	my $optref = $config{set_option} || [];
+	my @option = @$optref;					# pre-sorted option list
 
-	foreach my $opt ( sort keys %option ) {			# build sorted list
-		my $value = $option{$opt};
-		my @value = map { ref($_) ? @$_ : $_ } $value;
-		push @option, [$opt, $_] foreach @value;
+	my @expand;
+	while ( my $name = shift @option ) {			# expand option list
+		foreach my $value ( map { ref($_) ? @$_ : $_ } shift @option ) {
+			push @expand, [$name, $value];
+		}
 	}
 
 	my $format = ";; %s\t%s\n";
+	local $config{set_option} = \@expand;
 	foreach my $name ( sort keys %config ) {
-		local $config{set_option} = \@option;
 		my $value = $config{$name};
 		if ( ref $value ) {
 			foreach my $arg (@$value) {
@@ -520,16 +520,16 @@ sub _config {
 
 sub _option {
 	my ( $self, $name, @arg ) = @_;
-	my $entry = ( scalar(@arg) > 1 ) ? [@arg] : $arg[0];
-	my $opt	  = "${name}:";
+	my ($entry) = @arg;
+	my $opt = "${name}:";
 	$self->{test_ctx} = Net::DNS::Resolver::Unbound::Context->new() unless $self->{update};
 	$self->{test_ctx}->set_option( $opt, @arg ) if defined $entry;	  # error check only
 	my $updopt = $self->{update}->{set_option};
-	my %option = map {@$_} @$updopt;
+	my %option = map {$_} @$updopt;
 
 	unless (@arg) {
 		my $setopt = $self->{config}->{set_option};
-		my %config = map {@$_} @$setopt, @$updopt;
+		my %config = map {$_} @$setopt, @$updopt;
 		my $value  = $config{$opt};
 		return ref($value) ? @$value : $value;
 	}
@@ -539,11 +539,11 @@ sub _option {
 		$option{$opt} = $state = [$state] unless ref $state;
 		push @$state, $entry;
 	} else {						# initial entry
-		$option{$opt} = ref($entry) ? [$entry] : $entry;
+		$option{$opt} = $entry;
 	}
 
-	$option{$opt} = undef unless defined $entry;		# delete entire option
-	my @option = map { [$_, $option{$_}] } keys %option;
+	$option{$opt} = [] unless defined $entry;		# delete entire option
+	my @option = map { ( $_, $option{$_} ) } keys %option;
 	$self->{update}->{set_option} = \@option;
 	return;
 }
@@ -570,33 +570,26 @@ sub _finalise_config {
 	my $updopt = delete $update->{set_option};
 	my %config = ( %$config, %$update );			# merge config updates
 
-	foreach my $name ( keys %config ) {			# rebuild unbound context
-		my @element = map { ref($_) ? @$_ : $_ } $config{$name};
-		foreach my $element (@element) {
-			my @arg = map { ref($_) ? @$_ : $_ } $element;
-			$ctx->$name(@arg);
-		}
+	my %option = map {$_} @$cfgopt, @$updopt;		# merge option updates
+	delete @option{@IPpref};				# expunge IP preference
+	foreach my $key ( grep { $self->$_ } @IPconf ) {	# insert IP preference
+		my ( $key, $value ) = @{$IP_conf{$key}};
+		$option{$key} = $value;
+		last;
 	}
-
-	my %option = map {@$_} @$cfgopt, @$updopt;		# merge options
-	delete @option{@IPpref};				# expunge IP preferences
-	my @option;
+	my @option = map { ( $_, $option{$_} ) } sort keys %option;    # rebuild option list
 
 	foreach my $name ( keys %option ) {			# set unbound options
-		my $value = $option{$name} || next;
-		push @option, [$name, $value];
-		my @element = map { ref($_) ? @$_ : $_ } $value;
-		foreach my $element (@element) {
-			my @arg = map { ref($_) ? @$_ : $_ } $element;
-			$ctx->set_option( $name, @arg );
+		foreach my $value ( map { ref($_) ? @$_ : $_ } $option{$name} ) {
+			$ctx->set_option( $name, $value );
 		}
 	}
 
-	foreach my $key ( grep { $self->$_ } @IPconf ) {	# append IP preference
-		my $arg = $IP_conf{$key};
-		$ctx->set_option(@$arg);
-		push @option, $arg;
-		last;
+	foreach my $name ( keys %config ) {			# rebuild unbound context
+		foreach my $value ( map { ref($_) ? @$_ : $_ } $config{$name} ) {
+			my @arg = map { ref($_) ? @$_ : $_ } $value;
+			$ctx->$name(@arg);
+		}
 	}
 
 	$config{set_option} = \@option if @option;
